#include <allegro.h>
#include "051s.h"
#include "051sc.h"

int end_game;			/* flag we'll set to end the game */

BITMAP *dblbuffer;

CIRCLE_T *moving_circle;

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown(); 		/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    install_keyboard();

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	allegro_message("Couldn't set gfx mode: %s\n", allegro_error);
	exit(1);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory.");
	exit(2);
    }
    clear(dblbuffer);

    moving_circle = circle_init(SCREEN_W / 2, SCREEN_H / 2, 50, 3, 1, 0);
    if (moving_circle == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory.");
	exit(3);
    }
    circle_draw(moving_circle, dblbuffer);
    show_double_buffer();
}

void input(void)
{
    if (keypressed())
	end_game++;
}

void process(void)
{
    circle_update(moving_circle);
}

void output(void)
{
    circle_erase(moving_circle, dblbuffer);
    circle_draw(moving_circle, dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    circle_destroy(moving_circle);
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
