#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <allegro.h>
#include "0532uc.h"
#include "0532u.h"
#include "0532ul.h"

int end_game;			// flag we'll set to end the game

int how_many_circles;		// This contains the number of dinamic circles

int game_cycles;		// Elapsed game cycles

int active_circles;		// Active circles at the moment

int action;			// What are we doing? Adding or substracting?

BITMAP *dblbuffer;
CIRCLE_T *circles[MAX_CIRCLES];

int main(void)
{
    allegro_init();		// initialise the Allegro library

    init();			// initialise the game

    end_game = 0;		// reset flag

    do {			// loop

	input();		// get input

	process();		// process it

	output();		// give output

    } while (end_game == 0);	// until the flag is set

    shutdown(); 		// shut down anything that needs it

    allegro_exit();		// just for luck

    return 0;			// tell the OS that all went well

}
END_OF_MAIN()

void init(void)
{
    int i;

    srand(time(NULL));

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	allegro_message("Couldn't set gfx mode: %s\n", allegro_error);
	exit(1);
    }
    
    install_keyboard();
    install_timer();

    clear(screen);
    textout(screen, font, "How many circles you want to create?", 0, 0, 15);
    textout(screen, font, "Press a key from 1 to 9.", 0, 10, 15);
    textout(screen, font, "I will create [KEY] * 100 circles.", 0, 20, 15);
    textprintf(screen, font, 0, 30, 15, "The maximum is %d though...", MAX_CIRCLES - 1);

    how_many_circles = 0;
    while (how_many_circles == 0) {
	for (i = KEY_1; i <= KEY_9; i++) {
	    if (key[i])
		how_many_circles = (1 + i - KEY_1) * 100;
	}
    }
    clear_keybuf();

    if (how_many_circles < 1)
	how_many_circles = 1;
    if (how_many_circles > MAX_CIRCLES - 1)
	how_many_circles = MAX_CIRCLES - 1;

    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory.");
	exit(2);
    }
    clear(dblbuffer);

    // This inits the circle pointers...
    for (i = 0; i < MAX_CIRCLES; i++)
	circles[i] = NULL;
    // Now we create the dummy header...
    circles[0] = circle_init(0, 0, 0, 0, 0, 0, NULL);
    active_circles = 1;

    for (i = 0; i < how_many_circles; i++)
	add_circle();

    action = ADDING;
    game_cycles = 0;

    show_double_buffer();
}

void input(void)
{
    if (keypressed())
	end_game++;
}

void process(void)
{
    game_cycles++;

    if (game_cycles > 100) {
	switch (action) {
	case ADDING:
	    add_circle();
	    break;
	case SUBSTRACTING:
	    delete_circle();
	    break;
	}
	game_cycles = 0;
    }
    circle_update();
}

void output(void)
{
    /* Notice the following, we have to separate circle_erase from the other
     * loop, or we could erase circles while others were drawing themselves.
     */
    circle_erase(dblbuffer);

    circle_draw(dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    /* See, we have to substract one to the number of circles
       because we don't // want to count the dummy circle in. */

    textprintf(dblbuffer, font, 0, 0, 15, "Active circles: %d     ",
	       active_circles - 1);

    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
