// Init/shutdown routines...

#include <stdio.h>
#include <allegro.h>
#include "1032.h"
#include "1032init.h"
#include "1032tim.h"
#include "1032data.h"

void init_program(void)
{
    allegro_init();

    install_keyboard();
    install_timer();

    if (reserve_memory() != 0) {
	shutdown_program();
	allegro_message("Sorry, couldn't reserve enough memory.");
	exit(1);
    }
    if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) != 0) {
	allegro_message("Sorry, problems initializing sound...\n%s", allegro_error);
	exit(2);
    }
    if (load_our_data() != 0) {
	allegro_message("Sorry, couldn't load the data...");
	exit(3);
    }
    if (set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0) != 0) {
	allegro_message("Sorry, problems accessing mode-X 320x240...\n%s", allegro_error);
	exit(4);
    }
    // Write text to txt_buffer
    text_mode(-1);
    textout_centre(txt_buffer, font_object->dat, "Press ESC to exit",
		   SCREEN_W / 2, 0, makecol(0, 0, 0));
    textout_centre(txt_buffer, font_object->dat, "ENTER to show letter-monster",
		   SCREEN_W / 2, 22, makecol(0, 0, 0));

    // Blit background to buffer
    blit(background_object->dat, buffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

    clear(screen);
    set_palette(pal_object->dat);

    blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    play_midi(music_object->dat, 1000);

    init_custom_timers();

    monster_pos = 0;

    /* Now we calculate the speed of the monster. He has to appear for 4
     * long seconds, and therefore we use the formula:
     *
     * SCREEN_WIDTH+MONSTER_WIDTH / BEATS_PER_SECOND*4
     *
     * Remember BEATS_PER_SECOND is the number we initialized the timer.
     */
    add_pos = fdiv(itofix(SCREEN_W + 82), itofix(30 * 4));
}

int reserve_memory(void)
{
    buffer = create_bitmap(320, 240);
    txt_buffer = create_bitmap(320, 50);

    if (buffer == NULL || txt_buffer == NULL)
	return 1;		//Error

    // Clean up buffers
    clear(buffer);
    clear(txt_buffer);

    return 0;
}

void free_memory(void)
{
    destroy_bitmap(buffer);
    destroy_bitmap(txt_buffer);
}

void shutdown_program(void)
{
    free_memory();
    shutdown_timers();
    unload_our_data();

    allegro_exit();
    allegro_message("Bye bye...\n");
}
